import React, {Component} from 'react';
import PropTypes from 'prop-types';
import {
    Row,
    Col,
    Button,
    Card,
    CardTitle,
    CardBody,
    Alert
} from 'reactstrap';
import {map, find, size, filter} from 'lodash';
import './common/css/global.css'
import {
    TextField, CheckBoxField, GoogleCaptcha, SelectInput,
    TextareaField, DateField,
    File, SelectMultiple, TableView, MultipleInput, MultipleTextInput
} from './field/index';
import ValidatorService from './services/validator'
import {textCenter, titleStyle, linkStyle} from "./common/styles";

class Form extends Component {
    static propTypes = {
        siteKey: PropTypes.string.isRequired,
        isMobile: PropTypes.bool,
        error: PropTypes.shape({}).isRequired,
        chapter: PropTypes.shape({
            validateOnChange: PropTypes.func
        }).isRequired,
        onUpdateChapter: PropTypes.func.isRequired,
        values: PropTypes.shape().isRequired,
        chapterNumber: PropTypes.number.isRequired,
        allValues: PropTypes.arrayOf(PropTypes.shape()).isRequired
    };

    static defaultProps = {
        isMobile: false,
    };

    constructor(props) {
        super(props);
        this.state = {
            values: {
                tokenCaptcha: ''
            },
            searching: {},
            errors: {},
            showModal: false,
            isMobile: this.props.isMobile,
            plainFields: []
        };
        this.ref = null
   }

   setRef(ref){
        if(!this.ref){
            this.ref = ref;
        }
   }

   componentDidMount() {
        if (this.props.chapter.autoFocus === undefined || this.props.chapter.autoFocus === true) {
            if(this.ref){
                this.ref.current.focus();
            }
        }
    }

   handlerSetFocus(){
        if (this.props.chapter.autoFocus === undefined || this.props.chapter.autoFocus === true) {
            if (this.ref) {
                this.ref.current.focus();
            }
        }
    }

   getValue(key, isSelect) {
        if(isSelect){
            if(!this.props.values[key]){
                this.props.values[key] = {label: '[Seleccione]', value: null}
            }
        }
        return this.props.values[key] || '';
    }

   getField(fields) {
        if (!Array.isArray(fields)) {
            return this.switchField(fields);
        } else {
            return (
                <Row style={{marginRight: '-15px', marginLeft: '-15px'}}>
                    {
                        map(fields, field =>(
                            this.colAndSwitchField(field)
                        ))
                    }
                </Row>
            )
        }
    }

    colAndSwitchField(field){
        return (
            <Col>
                {this.switchField(field)}
            </Col>
        )
    }

    switchField(field) {
        const listColor = ['default', 'primary', 'success', 'danger', 'info', 'warning'];
        const {errors} = this.state;
        if (!errors[field.name]) {
            errors[field.name] = {
                status: null,
                message: ''
            };
        }
        this.state.errors = errors;

        this.state.plainFields.push(field);

        switch (field.type) {
            case 'button':
                if (field.isSubmit){
                    field.action ='isSubmit';
                }
                if (field.isSetChapter){
                    field.action = 'isSetChapter';
                }
                if (this.props.isMobile) {
                    return (
                        <Row>
                            <Col>
                                <Button
                                    style={listColor.indexOf(field.color.toLowerCase()) === -1 ?
                                        {
                                            backgroundColor: field.color,
                                            color: '#fff'
                                        }
                                        : null
                                    }
                                    id={field.name}
                                    name={field.name}
                                    color={listColor.indexOf(field.color.toLowerCase()) !== -1 ? field.color : null }
                                    onClick={e => this.handleOnclickButton(e)}
                                >
                                    {field.label}
                                </Button>
                            </Col>
                        </Row>
                    );
                }
                return (
                    <Button
                        style={listColor.indexOf(field.color.toLowerCase()) === -1 ?
                            {
                                backgroundColor: field.color,
                                color: '#fff'
                            }
                            : null
                        }
                        id={field.name}
                        name={field.name}
                        color={listColor.indexOf(field.color.toLowerCase()) !== -1 ? field.color : null }
                        onClick={e => this.handleOnclickButton(e)}
                    >
                        {field.label}
                    </Button>
                );
            case 'title':
                return (
                    <div style={titleStyle}>
                        {field.label}
                    </div>
                );
            case 'checkbox':
                return (
                    <CheckBoxField
                        inline={field.inline}
                        name={field.name}
                        label={field.label}
                        value={field.value == null ? this.getValue(field.name) : field.value}
                        type={field.type}
                        helpText={field.helpText}
                        disabled={field.disabled}
                        onChange={e => this.handleChange(e)}
                        error={this.state.errors[field.name]}
                        linkTo={field.linkTo}
                    />
                );
            case 'link':
                return (
                    <a
                        name={field.name}
                        href={field.url}
                        target={field.target}
                        style={linkStyle}
                    >
                        {field.label}
                    </a>
                );
            case 'multipleInput':
                return (
                    <MultipleInput
                        mode={field.mode ? field.mode : 'DEFAULT'}
                        name={field.name}
                        label={field.label}
                        placeholder={field.placeholder}
                        control={field.name}
                        value={this.getValue(field.name)}
                        type={field.subtype}
                        helpText={field.helpText}
                        disabled={field.disabled}
                        totalMessage={field.totalMessage}
                        onChange={e => this.handleChange(e, field.name)}
                        error={this.state.errors[field.name]}
                        onSetRef={ref => this.setRef(ref)}
                    />
                );
            case 'multipleTextInput':
                return (
                    <MultipleTextInput
                        name={field.name}
                        label={field.label}
                        placeholder={field.placeholder}
                        control={field.name}
                        value={this.getValue(field.name)}
                        helpText={field.helpText}
                        disabled={field.disabled}
                        onChange={e => this.handleChange(e, field.name)}
                        error={this.state.errors[field.name]}
                        onSetRef={ref => this.setRef(ref)}
                    />
                )
            case 'selectMultiple':
                return (
                    <SelectMultiple
                        source={field.source}
                        supportsWriteOthers={field.supportsWriteOthers}
                        required={false}
                        label={field.label}
                        control={field.name}
                        value={this.getValue(field.name, true)}
                        valueOthers={this.getValue(`${field.name}Others`)}
                        onChange={e => this.handleChange(e, field.name)}
                        onChangeOthers={e => this.handleChange(e, `${field.name}Others`)}
                        options={field.options}
                        helpText={field.helpText}
                        error={this.state.errors[field.name]}
                        manageError={e => this.manageErrors(field.name, e)}
                        placeholder={field.placeholder}
                        searchAmount={field.searchAmount}
                        token={field.token}
                    />
                );
            case 'table':
                return (
                    <TableView
                        table={field.table}
                        token={field.token}
                    />
                )
            case 'select':
                return (
                    <SelectInput
                        source={field.source}
                        supportsWriteOthers={field.supportsWriteOthers}
                        required={false}
                        label={field.label}
                        control={field.name}
                        value={this.getValue(field.name, true)}
                        valueOthers={this.getValue(`${field.name}Others`)}
                        onChange={e => this.handleChange(e, field.name)}
                        onChangeOthers={e => this.handleChange(e, `${field.name}Others`)}
                        onInputChange={e => this.handleInPutChange(e, field.name)}
                        options={field.options}
                        helpText={field.helpText}
                        error={this.state.errors[field.name]}
                        placeholder={field.placeholder}
                        searchAmount={field.searchAmount}
                        token={field.token}
                        onSetRef={ref => this.setRef(ref)}
                    />
                );
            case 'separator':
                return ' ';
            case 'rowseparator':
                return (
                    <Row>
                        <Col>
                            &nbsp;
                        </Col>
                    </Row>
                );
            case 'textarea':
                return (
                    <TextareaField
                        id={field.name}
                        name={field.name}
                        label={field.label}
                        placeholder={field.placeholder}
                        minLength={field.minLength}
                        maxLength={field.maxLength}
                        onChange={e => this.handleChange(e)}
                        value={field.value == null ? this.getValue(field.name) : field.value}
                        helpText={field.helpText}
                        error={this.state.errors[field.name]}
                    />
                );
            case 'numfield':
                return(
                    <TextField
                        name={field.name}
                        label={field.label}
                        placeholder={field.placeholder}
                        value={field.value == null ? this.getValue(field.name) : field.value}
                        type={field.type}
                        helpText={field.helpText}
                        disabled={field.disabled}
                        maxLength={field.maxlength ? field.maxlength : null}
                        minLength={field.minlength ? field.minlength : null}
                        onChange={e => this.handleChange(e)}
                        onKeyPress={e => this.validateNumberKeyPress(e, field)}
                        error={this.state.errors[field.name]}
                        onSetRef={ref => this.setRef(ref)}
                    />
                )
            case 'cuitfield':
                return(
                    <TextField
                        name={field.name}
                        label={field.label}
                        placeholder={field.placeholder}
                        value={field.value == null ? this.getValue(field.name) : field.value}
                        type={field.type}
                        helpText={field.helpText}
                        disabled={field.disabled}
                        maxLength="11"
                        minLength="11"
                        onChange={e => this.handleChange(e)}
                        onKeyPress={e => this.validateNumberKeyPress(e, field)}
                        error={this.state.errors[field.name]}
                        onSetRef={ref => this.setRef(ref)}
                    />
                )
            case 'phonefield':
                return (
                    <TextField
                        name={field.name}
                        label={field.label}
                        placeholder={field.placeholder}
                        value={field.value == null ? this.getValue(field.name) : field.value}
                        type={field.type}
                        helpText={field.helpText}
                        disabled={field.disabled}
                        onChange={e => this.handleChange(e)}
                        error={this.state.errors[field.name]}
                    />
                );
            case 'datefield':
                return (
                    <DateField
                        name={field.name}
                        label={field.label}
                        dateFormat={field.dateFormat}
                        value={field.value == null ? this.getValue(field.name) : field.value}
                        minDate={field.minDate}
                        maxDate={field.maxDate}
                        helpText={field.helpText}
                        onChange={e => this.handleChange(e)}
                        error={this.state.errors[field.name]}
                    />
                )
            case 'file':
                return (
                    <File
                        valuesForUpload={this.props.values[field.name]}
                        name={field.name}
                        label={field.label}
                        type={field.type}
                        helpText={field.helpText}
                        disabled={field.disabled}
                        onChange={e => this.onChangeHandlerFiles(e, field)}
                        onLoad={e => this.handlerSizePicture(e, field)}
                        error={this.state.errors[field.name]}
                    />
                )
            default:
                return (
                    <TextField
                        name={field.name}
                        label={field.label}
                        placeholder={field.placeholder}
                        value={field.value == null ? this.getValue(field.name) : field.value}
                        type={field.type}
                        helpText={field.helpText}
                        disabled={field.disabled}
                        onChange={e => this.handleChange(e)}
                        error={this.state.errors[field.name]}
                        onSetRef={ref => this.setRef(ref)}
                    />
                );
        }
    }

    handleInPutChange(e, name) {
        const source = find(this.state.plainFields, {name: name}).source;
        if (!source || source.length === 0) {
            const q =
                find(this.state.plainFields, {name: name}).searchAmount ?
                    find(this.props.chapter.fields, {name: name}).searchAmount
                    :
                    2;
            if (e !== undefined && size(e) > q) {
                if (this.timeout) clearTimeout(this.timeout);
                this.timeout = setTimeout(() => {
                    find(this.state.plainFields, {name: name}).onSubmit(e, name);
                }, 500);
            }
        }
    }

    getErrors(key) {
        return this.state.errors[key] || null;
    }

    handleErrorOnClickButton(field) {
        if (field.type !== 'link') {
            this.validate(field.name, field.validateOnChange, field.supportsWriteOthers);
            if (this.getErrors(field.name) === null) {
                this.validate(field.name, field.validate, field.supportsWriteOthers);
            } else {
                if (this.getErrors(field.name).status === null){
                    this.validate(field.name, field.validate, field.supportsWriteOthers);
                }
            }
        }
        if (this.props.chapter.showCaptcha) {
            if (this.state.values.tokenCaptcha === '') {
                this.manageErrors('tokenCaptcha',
                    {
                        status: 'invalid',
                        message: 'Debe validar el captcha'
                    }
                );
            } else {
                this.manageErrors('tokenCaptcha',
                    {
                        status: 'valid',
                        message: ''
                    }
                );
            }
        }
        if(field.type === 'file' && field.required){
            if(!this.props.values[field.name]){
                this.manageErrors(field.name,
                    {
                        status: 'invalid',
                        message: 'Este campo es requerido, ' + field.helpText
                    }
                );
            } else {
                if (this.props.values[field.name].length === 0){
                    this.manageErrors(field.name,
                        {
                            status: 'invalid',
                            message: 'Este campo es requerido, ' + field.helpText
                        }
                    );
                }
            }
        }
        if(field.type === 'numfield' && field.required){
            if(!this.props.values[field.name] || Number(this.props.values[field.name]) === 0){
                this.manageErrors(field.name,
                    {
                        status: 'invalid',
                        message: field.helpText ? 'Este campo es requerido, ' + field.helpText : 'Este campo es requerido'
                    }
                );
            }
        }
    }

    handleOnclickButton(e) {
        e.preventDefault();
        const button = find(this.props.chapter.buttons, {name: e.target.name});
        if (button.action === 'isSubmit') {
            map(this.state.plainFields, field => (
                this.handleErrorOnClickButton(field)
            ));
            this.props.error.anErrorOccurred = false;
            map(this.state.plainFields, field => (
                this.manageError(field)
            ));
            if (this.props.chapter.showCaptcha && this.state.values.tokenCaptcha === '') {
                this.props.error.anErrorOccurred = true;
                this.props.error.errorMsg = 'Se detectaron errores';
            }
            if (!this.props.error.anErrorOccurred) {
                this.props.values.chapterNumber = this.props.chapterNumber;
                this.props.values.actionButton = "isSubmit"
                if (this.props.chapter.id){
                    this.props.values.chapterId = this.props.chapter.id
                } else {
                    this.props.values.chapterId = 'wasNotInformed'
                }
                button.onSubmit(this.props.values, this.props.allValues);
            }
        }

        if (button.action === 'isSetChapter') {
            this.state.errors = {};
            this.props.error.anErrorOccurred = false;
            this.props.onUpdateChapter(button.chapterNumber, this.props.values);
        }

        if (button.action === 'isSetChapterWithValidation') {
            map(this.state.plainFields, field => (
                this.handleErrorOnClickButton(field)
            ));
            this.props.error.anErrorOccurred = false;
            map(this.state.plainFields, field => (
                this.manageError(field)
            ));
            if (this.props.chapter.showCaptcha && this.state.values.tokenCaptcha === '') {
                this.props.error.anErrorOccurred = true;
                this.props.error.errorMsg = 'Se detectaron errores';
            }
            if (!this.props.error.anErrorOccurred) {
                this.state.errors = {};
                this.props.error.anErrorOccurred = false;
                this.props.onUpdateChapter(button.chapterNumber, this.props.values);
            }
        }

        if (button.action === 'isCancel') {
            this.props.values.chapterNumber = this.props.chapterNumber;
            this.props.values.actionButton = "isCancel"
            if (this.props.chapter.id){
                this.props.values.chapterId = this.props.chapter.id
            } else {
                this.props.values.chapterId = 'wasNotInformed'
            }
            button.onSubmit(this.props.values, this.props.allValues);
        }

        if (button.executeSetFocus){
            this.handlerSetFocus();
        }
    }

    manageError(field) {
        if (field.type !== 'link') {
            this.props.error.anErrorOccurred = this.props.error.anErrorOccurred || this.getErrors(field.name).status === 'invalid';
        }
    }

    manageErrors(key, error) {
        const {errors} = this.state;
        if (!errors[key]) {
            errors[key] = {};
        }
        errors[key] = {status: error.status, message: error.message};
        if (error.status === 'invalid'){
            this.props.error.anErrorOccurred = true;
            this.props.error.errorMsg = 'Se detectaron errores';
        }
        this.setState({errors})
    }

    validate(key, validateFn, supportsWriteOthers) {
        if (validateFn) {
            if (supportsWriteOthers){
                this.manageErrors(key, validateFn(this.props.values[key], this.props.values[`${key}Others`]));
            } else {
                this.manageErrors(key, validateFn(this.props.values[key], this.props.values));
            }
        }
    }

    handlerSizePicture(e, field) {
        let msjError = '';
        if (field.width) {
            if(e.naturalWidth > field.width) {
                msjError = msjError + "El ancho maximo puede ser " + field.width + " pixeles. ";
            }
        }
        if (field.height) {
            if(e.naturalHeight > field.height) {
                msjError = msjError + "La altura maxima puede ser " + field.height + " pixeles.";
            }
        }
        if (msjError) {
            const {errors, values} = this.state;
            errors[field.name] = {status: 'invalid', message: msjError};
            values[field.name] = null;
            this.setState({values, errors});
        }
    }

    maxSelectFile(e, field) {
        const {files} = e.target;
        let max;
        if (field.maximumNumberOfFiles === undefined)
        {
            max = 1;
        } else {
            max = field.maximumNumberOfFiles;
        }
        if (files.length > max) {
            const {errors, values} = this.state;
            errors[field.name] = {status: 'invalid', message: 'Puede seleccionar Hasta ' + max +' archivo por vez\n${field.helpText}'};
            values[field.name] = null;
            this.setState({values, errors});
            return false;
        }
        return true;
    }

    checkMimeType(e, field) {
        const {files} = e.target;
        let x;
        for (x = 0; x < files.length; x += 1) {
            if (size(find(field.types, {type: files[x].type})) === 0) {
                const {errors, values} = this.state;
                errors[field.name] = {status: 'invalid', message: `${files[x].name} es un tipo no valido\n${field.helpText}`};
                values[field.name] = null;
                this.setState({values, errors});
                return false;
            }
        }
        return true;
    }

    checkFileSize(e, field) {
        const {files} = e.target;
        for (let x = 0; x < files.length; x += 1) {
            if (files[x].size > field.sizeFile) {
                const {errors, values} = this.state;
                errors[field.name] = {status: 'invalid', message: `${files[x].name} es un archivo muy grande, elija uno mas pequeño\n${field.helpText}`};
                values[field.name] = null;
                this.setState({values, errors});
                return false;
            }
        }
        return true;
    }

    onChangeHandlerFiles(e, field) {
        e.preventDefault();

        const {files} = e.target;

        if (this.maxSelectFile(e, field) &&
            this.checkMimeType(e, field) &&
            this.checkFileSize(e, field)
        ) {
            const {errors, values} = this.state;
            errors[field.name] = {status: 'valid', message: 'Tenes ' + files.length +' archivos incoporados'};
            values[field.name] = files;
            this.props.values[field.name] = files;
            this.setState({values, errors});
        }
    }

    handleChange(e, name) {
        this.props.error.anErrorOccurred = false;
        const {values} = this.props;
        let field;
        if (name && name.indexOf('Others') > 0){
            const nameOfSelect = name.substring(0, name.indexOf('Others'));
            field = find(this.state.plainFields,
                {name: nameOfSelect})
        } else{
            field = find(this.state.plainFields,
                name ? {name: name} : {name: e.target.name}
            );
        }
        if (field.type === 'checkbox') {
            values[e.target.name] = e.target.checked;
        } else {
            if (name && name.indexOf('Others') > 0){
                values[e.target.name] = e.target.value;
            } else {
                values[name ? name : e.target.name] = name ? e : e.target.value;
            }
        }

        if (field.validateOnChange) {
            this.validate(field.name, field.validateOnChange, field.supportsWriteOthers);
        } else {
            let error = {status: null, message: ''}
            if (field.type === 'cuitfield') {
                if (e.target.value !== '') {
                    if (ValidatorService.validateCuil(e.target.value)) {
                        error = {status: 'valid', message: 'Formato valido'}
                    } else {
                        error = {status: 'invalid', message: 'Formato invalido'}
                    }
                }
            }
            if (field.type === 'selectMultiple') {
                if (ValidatorService.validateSelectMultiple(e)) {
                    error = {status: 'valid', message: 'Formato valido'}
                } else {
                    error = {status: 'invalid', message: 'Formato invalido'}
                }
            }
            if (field.type === 'datefield') {
                if (e.target.value !== '') {
                    if (ValidatorService.validateDate(e.target.value)) {
                        error = {status: 'valid', message: 'Formato valido'}
                    } else {
                        error = {status: 'invalid', message: 'Formato invalido'}
                    }
                }
            }
            if (field.type === 'phonefield') {
                error = ValidatorService.validatePhone(e.target.value).error;
            }
            this.manageErrors(name ? name : e.target.name, error);
        }

        if(this.props.chapter.validateOnChange){
            map(this.props.chapter.fields, field => (
                this.handleErrorOnClickButton(field)
            ));
            this.props.error.anErrorOccurred = false;
            map(this.props.chapter.fields, field => (
                this.manageError(field)
            ));
            if (this.props.chapter.showCaptcha && this.state.values.tokenCaptcha === '') {
                this.props.error.anErrorOccurred = true;
                this.props.error.errorMsg = 'Se detectaron errores';
             }
            this.props.chapter.validateOnChange(this.state.errors, this.props.values);
        }
    }

    expiredCallback() {
        this.props.error.anErrorOccurred = true;
        const {values, errors} = this.state;
        values.tokenCaptcha = '';
        errors.tokenCaptcha = {
            status: 'invalid',
            message: 'El captcha expiro, vuelva a validarlo nuevamente'
        };
        this.props.values.tokenCaptcha = '';
        this.setState({values, errors});
    }

    handleVerifyCallback(e) {
        this.props.error.anErrorOccurred = false;
        const {errors} = this.state;
        this.props.values.tokenCaptcha = e;
        errors.tokenCaptcha = {
            status: 'valid',
            message: ''
        };
        const values = {
            tokenCaptcha: e
        };
        this.setState({values, errors});
    }

    validateNumberKeyPress(e, field) {
        let decimalPlaces;
        let negativeValues;
        if (field.type === 'cuitfield') {
            decimalPlaces = 0;
            negativeValues = false;
        } else {
            if (field.fix !== null && field.fix !== undefined) {
                decimalPlaces = field.fix;
            } else {
                decimalPlaces = 2;
            }
            if(field.negativeValues !== null && field.negativeValues !== undefined){
                negativeValues = field.negativeValues;
            } else {
                negativeValues = false;
            }
        }
        if (!ValidatorService.validateNumber(e.key, e, decimalPlaces, negativeValues)) {
            e.preventDefault();
            const {errors} = this.state;
            if (e.key === '.' && decimalPlaces === 0){
                errors[field.name] = {
                    status: 'invalid',
                    message: 'ATENCION --> no se admiten decimales'
                };
            } else {
                if(e.key === '-' && !negativeValues) {
                    errors[field.name] = {
                        status: 'invalid',
                        message: 'ATENCION --> no se admiten valores negativos'
                    };
                } else {
                    if (e.key === ',' && decimalPlaces > 0) {
                        errors[field.name] = {
                            status: 'invalid',
                            message: 'ATENCION --> para indicar decimales utilice el punto'
                        };
                    } else {
                        if (decimalPlaces > 0 ) {
                            if  (negativeValues)    {
                                errors[field.name] = {
                                    status: 'invalid',
                                    message: 'Solo acepta numeros positivos o negativos con ' + decimalPlaces + ' decimales'
                                };
                            } else {
                                errors[field.name] = {
                                    status: 'invalid',
                                    message: 'Solo acepta numeros positivos con ' + decimalPlaces + ' decimales'
                                };
                            }

                        } else {
                            if  (negativeValues)    {
                                errors[field.name] = {
                                    status: 'invalid',
                                    message: 'Solo acepta numeros positivos o negativos sin decimales'
                                };
                            } else {
                                errors[field.name] = {
                                    status: 'invalid',
                                    message: 'Solo acepta numeros positivos sin decimales'
                                };
                            }
                        }
                    }
                }
            }

            this.setState({errors});

            if (this.timeout) clearTimeout(this.timeout);
            this.timeout = setTimeout(() => {
                errors[field.name] = {
                    status: null,
                    message: ''
                };
                if (field.type === 'cuitfield'){
                    if(this.props.values[field.name] !== ''){
                        // si es un cuitfield y puso una letra es formato invalido sino es vacio
                        errors[field.name] = {status: 'invalid', message: 'Formato invalido'};
                    }
                }
                this.setState({errors});
            }, 600);
        }
    }

    render() {
        this.state.plainFields = [];
        const fieldsSelectWhitOthers = filter(this.props.chapter.fields,
            {
                'type': 'select',
                'supportsWriteOthers': true,
                'isSearching': false
            });
        map(fieldsSelectWhitOthers, field =>(
            !find(field.options, {value: 'idAnother'}) ?
            field.options.push({label: '[Otros]', value: 'idAnother'})
            :
            field.options
        ));
        return (
            <div>
                {!this.props.isMobile && (
                    <Card>
                        <CardBody>
                            {
                                this.props.chapter.title ?
                                    <CardTitle >
                                        <h2 >
                                            {this.props.chapter.title}
                                        </h2>
                                    </CardTitle> :
                                    null
                            }
                            <Row>
                                <Col>
                                    {
                                        map(this.props.chapter.fields, field => (
                                                this.getField(field)
                                            )
                                        )
                                    }
                                    <Row>
                                        <Col>
                                            &nbsp;
                                        </Col>
                                    </Row>
                                    {
                                        (this.props.chapter.showCaptcha
                                            && (
                                                <GoogleCaptcha
                                                    handleVerifyCallback={e => this.handleVerifyCallback(e)}
                                                    expiredCallback={() => this.expiredCallback()}
                                                    error={this.state.errors.tokenCaptcha}
                                                    siteKey={this.props.siteKey}
                                                />
                                            )
                                        )
                                    }
                                </Col>
                            </Row>
                            <Row>
                                <Col style={textCenter}>
                                    {map(this.props.chapter.buttons, button => (
                                        this.getField(button)
                                    ))}
                                </Col>
                            </Row>
                        </CardBody>
                    </Card>
                )}

                {this.props.isMobile && (
                    <div>

                            {
                                this.props.chapter.title ?
                                        <h2 >
                                            {this.props.chapter.title}
                                        </h2> :
                                    null
                            }
                            <div>
                                <Col>
                                    {
                                        map(this.props.chapter.fields, field => (
                                                this.getField(field)
                                            )
                                        )
                                    }
                                    <Row>
                                        <Col>
                                            &nbsp;
                                        </Col>
                                    </Row>
                                    {
                                        (this.props.chapter.showCaptcha
                                            && (
                                                <GoogleCaptcha
                                                    handleVerifyCallback={e => this.handleVerifyCallback(e)}
                                                    expiredCallback={() => this.expiredCallback()}
                                                    error={this.state.errors.tokenCaptcha}
                                                    siteKey={this.props.siteKey}
                                                />
                                            )
                                        )
                                    }
                                </Col>
                            </div>
                            <Row>
                                <Col>
                                    {map(this.props.chapter.buttons, button => (
                                        this.getField(button)
                                    ))}
                                </Col>
                            </Row>
                    </div>
                )}

                {this.props.error.anErrorOccurred
                && (
                    <Alert color="danger">
                        {this.props.error.errorMsg}
                    </Alert>
                )}
            </div>
        );
    }
}

export default Form;
