import React, {Component} from 'react';
import PropTypes from 'prop-types';
import {
    Label, Input, FormGroup, FormFeedback, UncontrolledTooltip, Button, InputGroup, InputGroupAddon
} from 'reactstrap';
import {
    small,
    flexR,
    noPadding,
    toolTip,
    padding,
    toolTipIcon,
    smallAlt,
    list,
    listItem,
    buttonWarning
} from '../common/styles/index';
import {map, remove, size} from "lodash";
import {FontAwesomeIcon} from '@fortawesome/react-fontawesome';
import {faPlusSquare} from '@fortawesome/free-solid-svg-icons'

class MultipleTextInput extends Component {
    static propTypes = {
        onChange: PropTypes.func.isRequired,
        name: PropTypes.string.isRequired,
        label: PropTypes.string.isRequired,
        value: PropTypes.string,
        placeholder: PropTypes.string,
        control: PropTypes.string.isRequired,
        helpText: PropTypes.string,
        disabled: PropTypes.bool,
        minLength: PropTypes.number,
        maxLength: PropTypes.number,
        error: PropTypes.shape(),
        onKeyPress: PropTypes.func,
        onSetRef: PropTypes.func
    };

    static defaultProps = {
        value: null,
        helpText: null,
        minLength: null,
        maxLength: null,
        error: {
            status: null
        },
        disabled: false,
        onKeyPress: null
    };

    constructor(props) {
        super(props);
        this.ref = React.createRef();
        this.state = {
            value: '',
            input_n: 0
        }
    }

    componentDidMount() {
        if (this.props.onSetRef) {
            this.props.onSetRef(this.ref);
        }
    }

    handleAdd() {
        let num = this.state.input_n
        const value = {label: this.state.value, value: "INPUT_"+num}

        if(this.state.value.length > 0 && this.state.value !== null){

           this.props.onChange([value].concat(this.props.value));
           num = num+1;
        }
        this.setState({value: [],input_n: num})
    }


    handleOnChange(e) {
        this.setState({value: e.target.value});
    }

    handleDelete(e) {
        let result = remove(this.props.value, (value, key) => {
                    return (key !== e)
                })
        size(result) === 0 ? this.props.onChange([]) : this.props.onChange(result)
    }

    render() {
        const {
            name, label, value, placeholder, type, helpText, disabled, error, minLength, maxLength, mode, totalMessage
        } = this.props;
        return (
            <FormGroup>
                <Label
                    style={{...small, ...noPadding, ...flexR, ...padding}}>
                    <p style={noPadding}>
                        {label + " "}
                    </p>
                    {helpText && (
                        <p style={{...noPadding, ...toolTipIcon, ...smallAlt}} id={'ref' + name}>
                            {String.fromCharCode(0x24D8)}
                        </p>
                    )}
                </Label>
                <InputGroup>
                    <Input
                        valid={error.status === 'valid' ? true : null}
                        invalid={error.status === 'invalid' ? true : null}
                        id={name}
                        placeholder={placeholder}
                        name={name}
                        type={type}
                        value={this.state.value}
                        minLength={minLength ? minLength : null}
                        maxLength={maxLength ? maxLength : null}
                        autoComplete="off"
                        disabled={disabled}
                        onChange={(e) => this.handleOnChange(e)}
                        onKeyDown={(e) => {
                            e.keyCode === 13 ? this.handleAdd() : null
                        }}
                        innerRef={this.ref}
                    />
                    <InputGroupAddon addonType="prepend">
                        <Button onClick={() => {
                            this.handleAdd()
                        }}>
                            <FontAwesomeIcon size={'sm'}
                                             icon={faPlusSquare}/>
                        </Button>
                    </InputGroupAddon>
                </InputGroup>
                <ul style={{...list}}>
                    <div>
                        {(value.value !== null) && (
                            map(value, ((value, key) => (
                                    <div key={key}
                                         style={{...listItem, ...{backgroundColor: (key % 2 === 0 ? 'lightgray' : 'white')}}}>
                                        <Label>{value.label}</Label>
                                        <Button stlye={{...buttonWarning}}
                                                color="danger" size="sm"
                                                onClick={() => this.handleDelete(key)}>x</Button>
                                    </div>
                                )
                            )))
                        }
                    </div>
                </ul>

                <FormFeedback valid={error.status === 'valid' ? true : null}>
                    {error.message}
                </FormFeedback>
                {helpText && (
                    <UncontrolledTooltip placement="top" target={'ref' + name}
                                         style={toolTip}>
                        {helpText}
                    </UncontrolledTooltip>
                )}
            </FormGroup>
        );
    }
}

export default MultipleTextInput;
